<?php

/**
 * Handles the Reference edit action.
 *
 * @author Marius Walk
 */

// TODO: Action class did not exist until MW 1.18
if ( ! class_exists( 'Action') ) {
	class Action{}
}

class RHEdit extends Action {

	/**
	 * Return the name of the action this object responds to
	 * @return String lowercase
	 */
	public function getName(){
		return 'rhedit';
	}

	/**
	 * Return array with name of each given service and its interface object.
	 * @return array
	 */
	public function getServices(){
		$mendeley = new RHMendeley();
		return array('mendeley.com' => $mendeley);
	}
	
	/**
	 * The main action entry point.
	 */
	public function show(){
		$title = $this->page->getTitle();
		// Make sure that this is not a special page, the correct category, and
		// that the user is allowed to edit it
		if ((!isset( $title )) ||
			($title->getNamespace() == NS_SPECIAL) ||
			 ($title->userCan( 'edit' ) == false) ||
			 (!RHHelper::activatedForCategory($title)) ) {
			return self::execute();
		}
		//reference helper has to be set up first. check if this is already done.
		$db = wfGetDB( DB_SLAVE );
		$template = $db->select(
			'reference_helper',
			'user',
			'field_type = "template"',
			__METHOD__,
			array( 'ORDER BY' => 'field_type' )
		);
		$category = $db->select(
			'reference_helper',
			'user',
			'field_type = "category"',
			__METHOD__,
			array( 'ORDER BY' => 'field_type' )
		);
		$property = $db->select(
			'reference_helper',
			'user',
			'field_type = "property"',
			__METHOD__,
			array( 'ORDER BY' => 'field_type' )
		);
		$variable = $db->select(
			'reference_helper',
			'user',
			'field_type = "variable"',
			__METHOD__,
			array( 'ORDER BY' => 'field_type' )
		);
		if ($template->numRows() == 0 ||
			$category->numRows() == 0 ||
			$variable->numRows() == 0) {
			return self::setUpFirst($this, $this->page);
		}


		//Make sure, which site has to be rendered
		//results already confirmerd? after pdf is parsed and user has confirmed
		if (isset($_POST['rh_edit_save'])) {
			return self::saveResults($this, $this->page);
		}

		//url for a pdf already given?
		if (!isset($_POST['rh_edit_search'])) {
			return self::renderForm($this, $this->page, ''); //return form for a url
		} else {
			$pdf = self::checkPDFSource($_POST['rh_edit_url']);
			if ($pdf === false) {
				return self::renderForm($this, $this->page, 'rh_edit_error_no_pdf');
			} else {
				$given_services = self::getServices();
				$selected_services = array();
				if (isset($_POST['rh_edit_services'])) {
					foreach ($_POST['rh_edit_services'] as $name) {
						array_push($selected_services, $given_services[$name]);
					}
				}
				return self::renderResults($this, $this->page, $pdf, $selected_services); //parse given pdf
			}
		}
	}

	/**
	 * Execute the action in a silent fashion: do not display anything or release any errors.
	 * @return Bool whether execution was successful
	 */
	public function execute(){
		return true;
	}

	/**
	 * Adds an "action" (a tab) to look up the references of that article
	 * @param $sktemplate
	 * @param $links list with all tabs
	 */
	static function displayTab($sktemplate, &$links ) {
		global $wgRequest, $wgUser;

		if ( method_exists ( $sktemplate, 'getTitle' ) ) {
			$title = $sktemplate->getTitle();
		} else {
			$title = $sktemplate->mTitle;
		}
		// Make sure that this is not a special page, the correct category, and
		// that the user is allowed to edit it
		if ((!isset( $title )) ||
			($title->getNamespace() == NS_SPECIAL) ||
			 ($title->userCan( 'edit' ) == false) || !RHHelper::activatedForCategory($title)) {
			return true;
		}
		if ( $wgUser->isAllowed( 'edit' )) {
			$update_references_text = 'rh_tab_edit';
			$create_tab = array(
			'class' => '',
			'text' => wfMessage($update_references_text)->text(),
			'href' => $title->getLocalURL( 'action=rhedit' ));

			$links['views']['rh_edit'] = $create_tab;
		}
		return true;
	}

	/**
	 * Gives back a form to enter a url to a pdf
	 * @param $action
	 * @param $article
	 * @param $error a string which is returned if there is no valid url
	 */
	static function renderForm( $action, $article, $error ) {
		$title = $article->getTitle();
		// For backward-compatibility
		if ( is_string( $action ) ) {
			global $wgOut;
			$output = $wgOut;
		} else {
			$output = $action->getOutput();
		}

		$output->setPageTitle(''.$title.' - '.wfMessage('rh_tab_edit').'');
		if ($error != '') {
			$output->addHTML("<strong>".wfMessage($error)."</strong>");
		}
		$output->addWikiMsg( 'rh_edit_url_description' );
		$output->addHTML('<form method="post" action="">
		<input type="text" size="45" name="rh_edit_url" value="http://example.com/example.pdf" />
		<br>'.wfMessage('rh_edit_service_selection').'<br>');
		foreach (self::getServices() as $name => $object) {
			$output->addHTML(''.$name.': <input type="checkbox" name="rh_edit_services[]" value="'.$name.'" />');
		}
		$output->addHTML('<br><input type="submit" name="rh_edit_search" value="'.wfMessage('rh_edit_button_search').'" />
		</form>');

		return true;
	}

	/**
	 * Checks if the given url points to a pdf file.
	 * @param $pdf_url path to a pdf
	 */
	static function checkPDFSource($pdf_url) {
		if (!file_exists($pdf_url)) {
			return false;
		}
		$url_type = substr($pdf_url, -3);
		if (strtolower($url_type) == 'pdf') {
			$pdf = tmpfile();
			$content = file_get_contents($pdf_url);
			fwrite($pdf, $content);
			$path = array_search('uri', @array_flip(stream_get_meta_data($GLOBALS[mt_rand()]=$pdf)));
			return $path;
		} else {
			return false;
		}
	}

	/**
	 * The function parses the given pdf and gives out the results.
	 * @param $action
	 * @param $article
	 * @param $pdf_url path to a pdf
	 * @param $services array of service objects to look for publication data
	 */
	static function renderResults($action, $article, $pdf_url, $services) {
		$title = $article->getTitle();
		// For backward-compatibility
		if ( is_string( $action ) ) {
			global $wgOut;
			$output = $wgOut;
		} else {
			$output = $action->getOutput();
		}

		$output->setPageTitle(''.$title.' - '.wfMessage('rh_tab_edit').'');

		$parser = new RHPDFParser($pdf_url);
		$publications = $parser->getCitatedPublications(); //returns array of RHPublication óbjects
		//give back the results in tables
		if (count($publications) > 0) {
			$output->addHTML('<p>'.wfMessage('rh_edit_results').'</p>');
			$pub_count = 0;
			$output->addHTML('<form method="post" action="">');
			foreach ($publications as $pub) {
				$pub->getInfoFromServices($services); //search for addition info on given services
				if ($pub->isValid()) {
				//return publications
					//get values in addition to citation, author, title
					$extra_values = $pub->getValues();
					unset($extra_values['title']);
					unset($extra_values['citation']);
					unset($extra_values['authors']);
					$extra_values_total = count($extra_values);
					$output->addHTML('<br><table width="700px">
										<tr>
										<td colspan="5"><strong>'.wfMessage('citation').':</strong> '.substr($pub->getCitation(), 0, 200).'</td>
										</tr>
										<tr>
										<td width="50px" rowspan="'.(ceil(($extra_values_total/2)) + 2).'">['.$pub_count.']<br><input type="checkbox" name="rh_edit_check['.$pub_count.']" value="'.$pub_count.'" checked></td>
										<td>'.wfMessage('title').': </td>
										<td colspan="3"><input type="text" name="rh_edit_title['.$pub_count.']" size="100" value="'.$pub->getTitle().'"></td>
										</tr>
										<tr>
										<td>'.wfMessage('authors').': </td>
										<td colspan="3"><input type="text" name="rh_edit_authors['.$pub_count.']" size="100" value="');
				//return authors seperated by comma
					$author_string = "";
					foreach ($pub->getAuthors() as $author) {
						$author_string .= ''.$author.', ';
					}
					$author_string = substr($author_string, 0, strlen($author_string) - 2);
					$output->addHTML(''.$author_string.'"></td>
										</tr>');
					$extra_value_counter = 0;
					//return extra values like for example year
					foreach ($extra_values as $key => $value) {
						$extra_value_counter++;
						if ($extra_value_counter % 2 == 1) {
							if ($extra_value_counter == $extra_values_total) {
								$output->addHTML('<tr><td>'.wfMessage($key).': </td>
									<td colspan="3">
									<input type="text" name="rh_edit_'.$key.'['.$pub_count.']" size="30" value="'.$value.'"></td></tr>');
							} else {
								$output->addHTML('<tr><td>'.wfMessage($key).': </td>
									<td><input type="text" name="rh_edit_'.$key.'['.$pub_count.']" size="30" value="'.$value.'"></td>');
							}
						} else {
							$output->addHTML('<td>'.wfMessage($key).': </td>
								<td><input type="text" name="rh_edit_'.$key.'['.$pub_count.']" size="30" value="'.$value.'"></td></tr>');
						}
					}
					$output->addHTML('</table>');	
					$output->addHTML('<p></p>');
					$pub_count++;	
				}
			}
			$output->addHTML('<input type="submit" name="rh_edit_save" value="'.wfMessage('rh_edit_button_save').'" /></form>');
		} else {
			$output->addHTML(wfMessage('rh_edit_error_no_results'));
		}

		return true;
	}

	/**
	 * The function saves the publications confirmed by user.
	 * @param $action
	 * @param $article
	 */
	static function saveResults( $action, $article) {
		$db = wfGetDB( DB_SLAVE );
		$title = $article->getTitle();
		// For backward-compatibility
		if ( is_string( $action ) ) {
			global $wgOut;
			$output = $wgOut;
		} else {
			$output = $action->getOutput();
		}
		$counter = 1;
		$references_text = '{{#set:';
		foreach ($_POST['rh_edit_check'] as $pub_nr) {
			$pub = new RHPublication('', array(), '');
			$pub_values = array();
			foreach ($pub->attributes as $attribute) {
				if ($attribute != 'authors') {
					if (isset($_POST['rh_edit_'.$attribute.''][$pub_nr])) {
						$pub_values[$attribute] = $_POST['rh_edit_'.$attribute.''][$pub_nr];
					}
				} else {
						$pub_values['authors'] = explode(",", $_POST['rh_edit_authors'][$pub_nr]);
				}
			}
			$pub->setValues($pub_values);
			//search wiki, if there is already an existing article for this publication
			$search_engine = new SearchMySQL($db);
			$search_results = $search_engine->searchTitle( $pub->getTitle() );
			if ($search_results != NULL && $search_results->numRows() > 0) {
				$search_result = $search_results->next();
				$pub_values['title'] = $search_result->getTitle();
				$pub->setValues($pub_values);
			} else {
				//create new wiki page
				$pub->saveAsWikiPage();
			}
			$references_text .= 'Has Reference = '.$pub->getTitle().'|';
			$counter++;
		}
		$references_text .= 'Reference Helper = 1';
		$references_text .= '}}';
		self::addReferencesToContent( $action, $article, $references_text );
		$output->setPageTitle($title);
		$output->addWikiText($article->getContent());
		return true;
	}


	/**
	 * Gives back information about the setup of this extension.
	 * @param $action
	 * @param $article
	 */
	static function addReferencesToContent( $action, $article, $references_text ) {
		global $wgUser;
		$content_text = $article->getContent();
		$content_text .= $references_text;
		MWDebug::log($content_text);
		$wikiPage = WikiPage::factory( $article->getTitle() );
		$content = new WikitextContent($content_text);
		//save wiki page
		$wikiPage->doEditContent( $content, 'ReferenceHelper added references', 'EDIT_UPDATE', false,
								   $wgUser, null );
		return true;
	}

	/**
	 * Gives back information about the setup of this extension.
	 * @param $action
	 * @param $article
	 */
	static function setUpFirst( $action, $article ) {
		$title = $article->getTitle();
		// For backward-compatibility
		if ( is_string( $action ) ) {
			global $wgOut;
			$output = $wgOut;
		} else {
			$output = $action->getOutput();
		}

		$output->setPageTitle(''.$title.' - '.wfMessage('rh_tab_edit').'');
		$output->addHTML("<strong>".wfMessage('rh_setup_needed')."");
		$output->addWikiText('[[special:rh_settings|'.wfMessage('rh_settings').']]');
		$output->addHTML("</strong>");
		return true;
	}

	/**
	 * A dummy method - this is needed for MediaWiki 1.18, where
	 * Action::getRestriction() is abstract and needs to be implemented.
	 */
	public function getRestriction() {
	}
}
