<?php

/**
 * Offers various static functions.
 * @author Marius Walk
 */
class RHHelper {

	/**
	 * Cleans a string. All special characters deleted and all upper case.
	 * @param $string string that should be cleande up
	 */
	static function cleanUp ($string) {
		$string = strtoupper($string);
		$string = preg_replace('/\\W/', '', $string);
		return $string;
	}

	/**
	 * Returns if ReferenceHelper is activated for the category
	 * of the current page.
	 * @param $title pages' title
	 * @param boolean
	 */
	static function activatedForCategory ($title) {
		//check category
		$db = wfGetDB( DB_SLAVE );
		$categories = $title->getParentCategories();
		$categories = array_keys($categories);
		foreach ($categories as $key => $name) {
			$name = substr($name, strrpos($name, ":") + 1);
			$categories[$key] = $name;
		}
		//get settings from db
		$category = $db->select(
					'reference_helper',
					array('user', 'standard'),
					'field_type = "category"',
					__METHOD__,
					array( 'ORDER BY' => 'field_type' )
				);
		if ($category->numRows() > 0) {
			$category_setting = $category->fetchRow()['user'];
			return in_array($category_setting, $categories);
		} else {
			return true;
		}
	}

	/**
	 * Returns html of a drop down menu.
	 * @param $option_values array with value of each option
	 * @param $selected_value value that should be selected
	 * @param $name name of the drop down menu
	 */
	static function createDropDownHtml($option_values, $selected_value, $name) {
		$dropdown = "";
			$select_body = "";
			foreach ( $option_values as $option ) {
				if ($selected_value == $option) {
					$select_body .= "	" . Html::element( 'option', array( 'value' => $option, 'selected' => true ), $option ) . "\n";
				} else {
					$select_body .= "	" . Html::element( 'option', array( 'value' => $option ), $option ) . "\n";
				}
			}
			$dropdown .= "\t" . Html::rawElement( 'select', array( 'name' => $name ), $select_body ) . "\n";
			return $dropdown;
	}

	/**
	 * Gets the text contents of a page with the passed-in Title object.
	 */
	static function getPageText( $title ) {
		if ( method_exists( 'WikiPage', 'getContent' ) ) {
			// MW 1.21+
			$wikiPage = new WikiPage( $title );
			$content = $wikiPage->getContent();

			if ( $content !== null ) {
				return $content->getNativeData();
			} else {
				return null;
			}
		} else {
			// MW <= 1.20
			$article = new Article( $title );
			return $article->getContent();
		}
	}
}

?>
