<?php

include_once 'RHServiceInterface.php';

/**
 * Defines the methods, a service must offer, that it can be used.
 * @author Marius Walk
 */
class RHMendeley implements RHServiceInterface {

	private $apiKey = '5ecac3410b982719bdc87844f417472105282822b';

	/**
	 * Creates a RHMendeley object.
	 * @return RHMendeley
	 */
	function __construct() {
		return $this;
	}

	/**
	 * Gives back an array of RHPublications objects by searching the service.
	 * @param $publication a RHPublication object
	 * @return array of RHPublication
	 */
	public function getSearchResults($publication) {
		$publications = array();
		//Pepare citation string
		$citation = $publication->getCitation();
		$citation_url = $citation;
		$citation_url = preg_replace('/\\W/', ' ', $citation_url); //delete special characters
		$citation_url = urlencode($citation_url);
		while ($citation_url != str_replace('++', '+', $citation_url)) {
			$citation_url = str_replace('++', '+', $citation_url);
		}
		//Consumer-Key for API
		$key = $this->apiKey;
		//Create URL to FreeCite API
		$url = 'http://api.mendeley.com/oapi/documents/search/'.$citation_url.'/?consumer_key='.$key.'';
		//data contains the parameters sent to FreeCite API
		$data = array('consumer_key', $key);
		//Execute HTTP-Request at FreeCite
		$ch = curl_init($url);
		curl_setopt($ch, CURLOPT_HTTPGET, true);
		curl_setopt($ch, CURLOPT_POST, FALSE);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		set_time_limit(0);
		$response = curl_exec($ch);
		if (curl_errno($ch) == 0) {
			$response = json_decode($response);
			curl_close($ch);
			if (isset($response->documents)) {
				$documents = $response->documents;
				$documents = array_slice($documents, 0, 5);
				foreach ($documents as $document) {
					$authors_unparsed = $document->authors;
					$authors = array();
					foreach ($authors_unparsed as $author) {
						array_push($authors, ''.substr((string) $author->forename, 0,1).'. '.(string)  $author->surname.'');
					}
					$pub = new RHPublication($document->title, $authors, $citation);
					$extra_values = array();
					$extra_values['year'] = $document->year;
					$extra_values['url'] = $document->mendeley_url;
					$extra_values['doi'] = $document->doi;
					$extra_values['journal'] = $document->publication_outlet;
					$pub->setValues($extra_values);
					array_push($publications, $pub);
				}
			}
			return $publications;
		} else {
			echo 'timeout';
			array_push($publications, $publication);
		}
		return $publications;
	}
}

?>
