<?php

/**
 * Creates the special page for RHGraph and handles its actions.
 *
 * @author Marius Walk
 */

class RHGraph extends SpecialPage {
	
	function __construct() {
		parent::__construct( 'RH_Graph' );
	}
	
	protected $m_querystring = '';
	protected $m_params = array();
	protected $m_printouts = array();
	protected $m_editquery = false;

	protected $params = array();

	/**
	 * Main mehtod, called when page is requested.
	 */
	function execute( $par ) {
		global $wgOut, $wgRequest, $smwgQMaxInlineLimit;
		
		//reference helper has to be set up first. check if this is already done.
		$db = wfGetDB( DB_SLAVE );
		$template = $db->select(
			'reference_helper',
			'user',
			'field_type = "template"',
			__METHOD__,
			array( 'ORDER BY' => 'field_type' )
		);
		$category = $db->select(
			'reference_helper',
			'user',
			'field_type = "category"',
			__METHOD__,
			array( 'ORDER BY' => 'field_type' )
		);
		$property = $db->select(
			'reference_helper',
			'user',
			'field_type = "property"',
			__METHOD__,
			array( 'ORDER BY' => 'field_type' )
		);
		$variable = $db->select(
			'reference_helper',
			'user',
			'field_type = "variable"',
			__METHOD__,
			array( 'ORDER BY' => 'field_type' )
		);
		if ($template->numRows() == 0 ||
			$category->numRows() == 0 ||
			$variable->numRows() == 0) {
				$wgOut->setPageTitle( 'ReferenceHelper - '.wfMessage( 'rh_graph' )->text().'' );
				$wgOut->addHTML("<strong>".wfMessage('rh_setup_needed')."");
				$wgOut->addWikiText('[[special:rh_settings|'.wfMessage('rh_settings').']]');
				$wgOut->addHTML("</strong>");
				return true;
		}

		//Set title of the special Page
		$wgOut->setPageTitle( 'ReferenceHelper - '.wfMessage( 'rh_graph' )->text().'' );
		$wgOut->addHTML(wfMessage('rh_graph_text'));
		$category = $category->fetchRow()['user'];


		//check if form was sent
		$condition = '';
		if ($wgRequest->getVal('rh_graph_submit') != NULL) {
			$condition = $wgRequest->getVal('rh_graph_condition');
			$position = strpos($condition, ']]');
			//check if condition is valid
			if ($position !== false || $condition == '') {
				if ($condition != '') {
					$condition = substr($condition, 0, $position + 2);
				}
				//print out form and graph
				$this->showForm($category, $condition);
				$this->showGraph($category, $condition);
			} else {
				//print out error and form
				$this->showForm($category, $condition);
				$wgOut->addHTML(wfMessage('rh_graph_wrong_condition'));
			}
		} else {
			$this->showForm($category, $condition);
		}

		return true;
	}


	/**
	 * Include protovis (visualization tool) library which draws the graph.
	 */
	public static function onBeforePageDisplay( OutputPage &$out, Skin &$skin ) {
		global $wgExtensionAssetsPath;
		//Sources for Arc Diagram
		$path = ''.$wgExtensionAssetsPath.'/ReferenceHelper/libs/protovis/protovis.js';
		$html = '<script type="text/javascript" src="'.$path.'"></script>';
		$out->addScript( $html );
		//Sources for directed graph
		$path = ''.$wgExtensionAssetsPath.'/ReferenceHelper/libs/d3/d3.js';
		$html = '<script type="text/javascript" src="'.$path.'"></script>';
		$out->addScript( $html );
		$src = ''.$wgExtensionAssetsPath.'/ReferenceHelper/libs/d3/app.css';
		$out->addStyle($src);
		return true;
	}

	/**
	 * Prints out the graph from given category and condition.
	 * @param $category string
	 * @param $condition string
	 */
	private function showGraph( $category, $condition ) {
		global $wgOut;
		$graph_data = $this->getResult($category, $condition);
		$wgOut->addHTML('<div><strong>'.wfMessage('rh_graph_directed').'</strong></div>');
		$wgOut->addHTML($this->getDirectedGraph($graph_data));
		$wgOut->addHTML('<div><strong>'.wfMessage('rh_graph_arc').'</strong></div>');
		$wgOut->addHTML($this->getArcDiagram($graph_data));
		return true;
	}

	/**
	 * Generates the form to select publications.
	 * @param $category string
	 * @param $condition string
	 */
	private function showForm($category, $condition) {
		global $wgOut;
		$return = '';
		$return .= '<br><form method="post" action="">';
		$return .= '<strong>'.wfMessage('rh_graph_category').':</strong> '.$category.' --> ';
		$return .= '<strong>'.wfMessage('rh_graph_condition').':</strong> <input type="text" name="rh_graph_condition" value="'.$condition.'" />';
		$return .= '<input type="submit" name="rh_graph_submit" value="'.wfMessage('rh_graph_button').'" />';
		$return .= '</form>';
		$wgOut->addHTML($return);
		return true;
	}

	/**
	 * Generates an array with all requested publications and their references.
	 * @param $category string
	 * @param $condition string
	 * @return array
	 */
	private function getResult($category, $condition) {
		global $wgOut;
		//querystring
		$q = '{{#ask: [[Category:'.$category.']] '.$condition.' }}';
		//values
		$po = 'Has_Reference = Referenz';
		//params
		$p = array();
		$p['format'] = 'list';
		$p['link']='all';
		$p['headers']='show';
		$p['mainlabel']='test';
		$p['limit']=200;
		
		$rawparams = SMWInfolink::decodeParameters( $p, false );
		
		// Check for q= query string, used whenever this special page calls itself (via submit or plain link):
		$this->m_querystring = $q;
		$rawparams[] = $this->m_querystring;

		// Check for param strings in po (printouts), appears in some links and in submits:
		$paramstring = $po;

		$ps = explode( "\n", $paramstring ); // params separated by newlines here (compatible with text-input for printouts)

		foreach ( $ps as $param ) { // add initial ? if omitted (all params considered as printouts)
			$param = trim( $param );

			if ( ( $param !== '' ) && ( $param { 0 } != '?' ) ) {
					$param = '?' . $param;
			}
			$rawparams[] = $param;
		}

		// Now parse parameters and rebuilt the param strings for URLs.
		list( $this->m_querystring, $this->m_params, $this->m_printouts ) = SMWQueryProcessor::getComponentsFromFunctionParams( $rawparams, false );

		$params = SMWQueryProcessor::getProcessedParams( $this->m_params, $this->m_printouts );
		$this->m_params['format'] = $params['format']->getValue();

		$this->params = $params;
		$queryobj = SMWQueryProcessor::createQuery(
				$this->m_querystring,
				$params,
				SMWQueryProcessor::SPECIAL_PAGE ,
				$this->m_params['format'],
				$this->m_printouts
		);

		/**
		 * @var SMWQueryResult $res
		*/

		// Determine query results
		$res = $params['source']->getValue()->getQueryResult( $queryobj );

		if ($res->getCount() < 1) {
			$wgOut->addHTML('<p>'.wfMessage('rh_graph_no_results').'</p>');
			return false;
		}

		$result_array = $res->serializeToArray();
		$result_array = $result_array['results'];

		$graph_data = array();

		foreach ($result_array as $title => $value) {
			$graph_data[$title] = array();
			foreach ($value['printouts']['Referenz'] as $reference) {
				array_push($graph_data[$title], $reference['fulltext']);
			}
		}

		return $graph_data;
	}

	/**
	 * Generates html-code to display an arc diagram.
	 * @param $graph_data array with all publications and their references
	 * @return $return html code to dislpay the graph
	 */
	private function getArcDiagram($graph_data) {
			$graph_data_titles = array_keys($graph_data);

			$return = '
			<script type="text/javascript+protovis">
						var graph_data = {
			  nodes:[';
				$nodes = '';
				foreach ($graph_data_titles as $title) {
					$nodes .= '{nodeName:"'.$title.'", group:1},';
				}
				$return .= $nodes;
			 $return .= '],
			  links:[';
			  	$source = 0;
			  	$links = '';
			  	foreach ($graph_data as $title => $references) {
			  		foreach ($references as $reference) {
			  			$target = array_search($reference, $graph_data_titles);
			  			if ($target !== false && !($reference == $graph_data_titles[$source])) {
			  				MWDebug::log('Title: '.$graph_data_titles[$source].' --> Target: '.$graph_data_titles[$target].'');
			  				$links .= '{source:'.$source.', target:'.$target.', value:1},';
			  			}
			  		}
			  		$source++;
			  	}
			  	$return .= $links;
			    $return .= '
			  ]
			};
			    					</script>';

					$return .= '<script type="text/javascript+protovis">
										var vis = new pv.Panel()
										    .width(1200)
										    .height(750)
										    .bottom(300);

										var arc = vis.add(pv.Layout.Arc)
										    .nodes(graph_data.nodes)
										    .links(graph_data.links)
										    .sort(function(a, b) a.group == b.group
										        ? b.linkDegree - a.linkDegree
										        : b.group - a.group);

										arc.link.add(pv.Line);

										arc.node.add(pv.Dot)
										    .size(function(d) (d.linkDegree + 1) * 20)
										    .fillStyle(pv.Colors.category19().by(function(d) d.group))
										    .strokeStyle(function() this.fillStyle().darker())
										    .title(function(d) "(" + d.linkDegree + ") " + d.nodeName)
										    .cursor("pointer");

										arc.label.add(pv.Label)

										vis.render();

										    </script>';
			return $return;
	}

	/**
	 * Generates html-code to display a directed graph.
	 * @param $graph_data array with all publications and their references
	 * @return $return html code to dislpay the graph
	 */
	private function getDirectedGraph($graph_data) {
		global $wgExtensionAssetsPath;
			$graph_data_titles = array_keys($graph_data);

			$return = '
			<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" id="svg"></svg>
			<script src="'.$wgExtensionAssetsPath.'/ReferenceHelper/libs/d3/d3.js"></script>
					<script>

					// http://blog.thomsonreuters.com/index.php/mobile-patent-suits-graphic-of-the-day/
					var links = [';

					foreach ($graph_data as $title => $references) {
						foreach ($references as $reference) {
							$return .= '{source: "'.$title.'", target: "'.$reference.'", type: "licensing"},';
						}
					}

					  $return .= '
					];

					var nodes = {};

					// Compute the distinct nodes from the links.
					links.forEach(function(link) {
					  link.source = nodes[link.source] || (nodes[link.source] = {name: link.source});
					  link.target = nodes[link.target] || (nodes[link.target] = {name: link.target});
					});

					var width = 1200,
					    height = 500;

					var force = d3.layout.force()
					    .nodes(d3.values(nodes))
					    .links(links)
					    .size([width, height])
					    .linkDistance(60)
					    .charge(-300)
					    .on("tick", tick)
					    .start();

					var svg = d3.select("svg")
					    .attr("width", width)
					    .attr("height", height);

					// Per-type markers, as they dont inherit styles.
					svg.append("defs").selectAll("marker")
					    .data(["suit", "licensing", "resolved"])
					  .enter().append("marker")
					    .attr("id", function(d) { return d; })
					    .attr("viewBox", "0 -5 10 10")
					    .attr("refX", 15)
					    .attr("refY", -1.5)
					    .attr("markerWidth", 6)
					    .attr("markerHeight", 6)
					    .attr("orient", "auto")
					  .append("path")
					    .attr("d", "M0,-5L10,0L0,5");

					var path = svg.append("g").selectAll("path")
					    .data(force.links())
					  .enter().append("path")
					    .attr("class", function(d) { return "dialink " + d.type; })
					    .attr("marker-end", function(d) { return "url(#" + d.type + ")"; });

					var circle = svg.append("g").selectAll("circle")
					    .data(force.nodes())
					  .enter().append("circle")
					    .attr("r", 6)
					    .call(force.drag);

					 svg.selectAll("circle")
   					.append("svg:title")
   					.text(function(d) { return d.name; });

					var text = svg.append("g").selectAll("text")
					    .data(force.nodes())
					  	.enter().append("text")
					    .attr("x", 8)
					    .attr("y", ".31em")


					// Use elliptical arc path segments to doubly-encode directionality.
					function tick() {
					  path.attr("d", linkArc);
					  circle.attr("transform", transform);
					  text.attr("transform", transform);
					}

					function linkArc(d) {
					  var dx = d.target.x - d.source.x,
					      dy = d.target.y - d.source.y,
					      dr = Math.sqrt(dx * dx + dy * dy);
					  return "M" + d.source.x + "," + d.source.y + "A" + dr + "," + dr + " 0 0,1 " + d.target.x + "," + d.target.y;
					}

					function transform(d) {
					  return "translate(" + d.x + "," + d.y + ")";
					}

					</script>';
			return $return;
	}
}
