<?php

/**
 * Creates the special page for RHSettings and handles its actions.
 *
 * @author Marius Walk
 */
class RHSettings extends SpecialPage {
	
	function __construct() {
		parent::__construct( 'RH_Settings' );
	}
	
	/**
	 * Main mehtod, called when page is requested.
	 */
	function execute( $par ) {
		global $wgOut, $wgRequest;

		//check if user wants to save data
		if (isset($_POST['rh_settings_category_submit'])) {
			$this->saveCategory($_POST['rh_settings_category']);
		}
		if (isset($_POST['rh_settings_template_submit'])) {
			$this->saveTemplate($_POST['rh_settings_template']);
		}
		if (isset($_POST['rh_settings_variables_submit'])) {
			$this->saveVariables($_POST);
		}

		//get settings from db
		$db = wfGetDB( DB_SLAVE );
		$template = $db->select(
			'reference_helper',
			array('user', 'standard'),
			'field_type = "template"',
			__METHOD__,
			array( 'ORDER BY' => 'field_type' )
		);
		$category = $db->select(
			'reference_helper',
			array('user', 'standard'),
			'field_type = "category"',
			__METHOD__,
			array( 'ORDER BY' => 'field_type' )
		);
		$property = $db->select(
			'reference_helper',
			array('user', 'standard'),
			'field_type = "property"',
			__METHOD__,
			array( 'ORDER BY' => 'field_type' )
		);
		$variable = $db->select(
			'reference_helper',
			array('user', 'standard'),
			'field_type = "variable"',
			__METHOD__,
			array( 'ORDER BY' => 'field_type' )
		);
		if ($category->numRows() < 1) {
			$wgOut->addHTML('<p><strong>'.wfMessage('rh_settings_category_missing').'</strong></p>');
		}
		if ($template->numRows() < 1) {
			$wgOut->addHTML('<p><strong>'.wfMessage('rh_settings_template_missing').'</strong></p>');
		}
		if ($variable->numRows() < 1) {
			$wgOut->addHTML('<p><strong>'.wfMessage('rh_settings_variable_missing').'</strong></p>');
		}

		//Set title of the special Page
		$wgOut->setPageTitle( 'ReferenceHelper - '.wfMessage( 'rh_settings' )->text().'' );

		//Create content of the special Page
		$category_section = $this->getCategorySection($category);
		$wgOut->addHTML($category_section);

		$wgOut->addHTML('<p></p>');

		$template_section = $this->getTemplateSection($template);
		$wgOut->addHTML($template_section);

		$wgOut->addHTML('<p></p>');

		$variable_section = $this->getVariableSection($variable);
		$wgOut->addHTML($variable_section);
	}

	/**
	 * Creates html for the Template form.
	 * @param $template_settings current template settings from database
	 */
	private function getTemplateSection($template_settings) {
		$db = wfGetDB( DB_SLAVE );
		// Get the names of all templates on this site.
		// Create form to chose a template
		$return = '';
		$return .= wfMessage('rh_settings_template_text');
		$return .= '<br><form method="post" action="">';
		$template_res = $db->select(
			'page',
			'page_title',
			array( 'page_namespace' => NS_TEMPLATE, 'page_is_redirect' => 0 ),
			__METHOD__,
			array( 'ORDER BY' => 'page_title' )
		);


		//get current template settings
		$current = '';
		if ($template_settings->numRows() == 1) {
			$current = $template_settings->fetchRow()['user'];
		}

		if ( $db->numRows( $template_res ) > 0 ) {
			$drop_down_options = array();
			foreach ($template_res as $template) {
				array_push($drop_down_options, $template->page_title);
			}
			$template_dropdown = RHHelper::createDropDownHtml($drop_down_options, $current, 'rh_settings_template');
			$return .=  $template_dropdown;
		}
		$return .= '<input type="submit" name="rh_settings_template_submit" value="'.wfMessage('rh_settings_template_button').'" />';
		$return .= '</form>';
		return $return;
	}

	/**
	 * Creates html for the category form.
	 * @param $category_settings current category settings from database
	 */
	private function getCategorySection($category_settings) {
		$db = wfGetDB( DB_SLAVE );
		// Get the names of all categories on this site.
		// Create form to chose a category
		$return = '';
		$return .= wfMessage('rh_settings_category_text');
		$return .= '<br><form method="post" action="">';
		$category_res = $db->select(
			'category',
			'cat_title',
			'',
			__METHOD__,
			array( 'ORDER BY' => 'cat_title' )
		);

		//get current category settings
		$current = '';
		if ($category_settings->numRows() == 1) {
			$current = $category_settings->fetchRow()['user'];
		}

		if ( $db->numRows( $category_res ) > 0 ) {
			$drop_down_options = array();
			foreach ($category_res as $category) {
				array_push($drop_down_options, $category->cat_title);
			}
			$category_dropdown = RHHelper::createDropDownHtml($drop_down_options, $current, 'rh_settings_category');
			$return .=  $category_dropdown;
		}
		$return .= '<input type="submit" name="rh_settings_category_submit" value="'.wfMessage('rh_settings_template_button').'" />';
		$return .= '</form>';
		return $return;
	}

	/**
	 * Creates html for the variables form.
	 * @param $variable_settings current variables settings from database
	 */
	private function getVariableSection($variable_settings) {
		$db = wfGetDB( DB_SLAVE );
		$template = $db->select(
			'reference_helper',
			array('user', 'standard'),
			'field_type = "template"',
			__METHOD__,
			array( 'ORDER BY' => 'field_type' )
		);
		//prepare current settings in array['user'] = standard
		$current_vars = array();
		foreach ($variable_settings as $variable_setting) {
			$current_vars[$variable_setting->user] = $variable_setting->standard;
		}
		//get current template settings
		$current_template = '';
		if ($template->numRows() == 1) {
			$current_template = $template->fetchRow()['user'];
			$template_title = Title::makeTitleSafe( NS_TEMPLATE, $current_template);
			//get vars from template
			if ( isset( $template_title ) ) {
				$return = '';
				$return .= wfMessage('rh_settings_variable_text');
				$return .= '<br><form method="post" action="">';
				$templateText = RHHelper::getPageText( $template_title );
				$vars = array();
				if ( preg_match_all( '/{{{([^|}]*)/mis', $templateText, $matches ) ) {
					foreach ( $matches[1] as $fieldName ) {
						$fieldName = trim( $fieldName );
						array_push($vars, $fieldName);
					}
				}
				$pub = new RHPublication('', array(), '');
				$infos = $pub->attributes;
				array_push($infos, '');
				sort($vars);
				sort($infos);
				$vars = array_unique($vars);
				$return .= '<table width="400px">';
				foreach ($vars as $name) {
					$current = '';
					if (isset($current_vars[$name])) {
						$current = $current_vars[$name];
					}
					$return .= '<tr><td>'.$name.':</td><td>'.RHHelper::createDropDownHtml($infos, $current, 'rh_settings_variable_'.$name.'').'</td></tr>';
				}
			}
			$return .= '</table>';
			$return .= '<input type="submit" name="rh_settings_variables_submit" value="'.wfMessage('rh_settings_template_button').'" />';
			$return .= '</form>';
		return $return;
		} else {
			return '';
		}
	}

	/**
	 * Saves the selected category to database.
	 * @param $category name of selected category
	 */
	private function saveCategory($category) {
		$db = wfGetDB( DB_SLAVE );
		$db->delete( 'reference_helper', array('field_type = "category"'), __METHOD__ );
		$db->insert( 'reference_helper',
			array('field_type' => 'category', 'user' => $category),
			__METHOD__, $options = array() );
		return true;
	}

	/**
	 * Saves the selected Template to database.
	 * @param $new_template name of selected template
	 */
	private function saveTemplate($new_template) {
		$db = wfGetDB( DB_SLAVE );
		$old_template = $db->select(
			'reference_helper',
			array('user', 'standard'),
			'field_type = "template"',
			__METHOD__,
			array( 'ORDER BY' => 'field_type' )
		);
		//check if current and old template are the same
		//if not, current variable settings have to be deleted
		if ($old_template->numRows() < 1) {
			$db->insert( 'reference_helper',
			array('field_type' => 'template', 'user' => $new_template),
			__METHOD__, $options = array() );
		} else {
			if ($old_template->fetchRow()['user'] != $new_template) {
				$db->delete( 'reference_helper', array('field_type = "template" OR field_type = "variable"'), __METHOD__ );
				$db->insert( 'reference_helper',
			array('field_type' => 'template', 'user' => $new_template),
			__METHOD__, $options = array() );
			}
		}
		return true;
	}

	/**
	 * Saves the selected variable settings to database.
	 * @param $vars
	 */
	private function saveVariables($vars) {
		$db = wfGetDB( DB_SLAVE );
		$db->delete( 'reference_helper', array('field_type = "variable"'), __METHOD__ );
		foreach ($vars as $user => $standard) {
			if (strpos($user, "rh_settings_variable_") !== false && $standard != '') {
				$user_value = substr($user, 21);
				$standard_value = $standard;
				$db->insert( 'reference_helper',
				array('field_type' => 'variable', 'user' => $user_value, 'standard' => $standard_value),
				__METHOD__, $options = array() );
			}
		}
		return true;
	}
}
